@@ -10,18 +10,20 @@
 from datetime import date
 from dateutil.relativedelta import relativedelta
 
-from trytond.model import ModelSQL, ModelView
 from trytond.transaction import Transaction
-from trytond.pool import Pool
+from trytond.pool import PoolMeta
 from nereid import request, cache, jsonify, abort
 from nereid.helpers import key_from_list
 
+__all__ = ['Product']
+__metaclass__ = PoolMeta
 
-class Product(ModelSQL, ModelView):
+
+class Product:
     "Product extension for Nereid"
-    _name = "product.product"
+    __name__ = "product.product"
 
-    def sale_price(self, product, quantity=0):
+    def sale_price(self, quantity=0):
         """Return the Sales Price.
         A wrapper designed to work as a context variable in templating
 
@@ -32,9 +34,8 @@ class Product(ModelSQL, ModelView):
 
         Finally if neither the guest user, nor the regsitered user has a
         pricelist set against them then the list price is displayed as the
-        price of the product
+        list price of the product
 
-        :param product: ID of product
         :param quantity: Quantity
         """
         price_list = request.nereid_user.sale_price_list.id if \
@@ -43,7 +44,6 @@ class Product(ModelSQL, ModelView):
         # If the registered user does not have a pricelist try for
         # the pricelist of guest user
         if not request.is_guest_user and price_list is None:
-            user_obj = Pool().get('nereid.user')
             guest_user = request.nereid_website.guest_user
             price_list = guest_user.sale_price_list.id if \
                 guest_user.sale_price_list else None
@@ -53,24 +53,24 @@ class Product(ModelSQL, ModelView):
             Transaction().cursor.dbname,
             Transaction().user,
             request.nereid_user.party.id,
-            price_list, product, quantity,
+            price_list, self.id, quantity,
             request.nereid_currency.id,
             'product.product.sale_price',
             ])
-        rv = cache.get(cache_key)
-        if rv is None:
+        price = cache.get(cache_key)
+        if price is None:
             # There is a valid pricelist, now get the price
             with Transaction().set_context(
                     customer = request.nereid_user.party.id,
                     price_list = price_list,
                     currency = request.nereid_currency.id):
-                rv = self.get_sale_price([product], quantity)[product]
+                price = self.get_sale_price([self], quantity)[self.id]
 
             # Now convert the price to the session currency
-            cache.set(cache_key, rv, 60 * 5)
-        return rv
+            cache.set(cache_key, price, 60 * 5)
+        return price
 
-    def get_availability(self, product):
+    def get_availability(self):
         """
         This method could be subclassed to implement your custom availability
         behavior.
@@ -83,7 +83,6 @@ class Product(ModelSQL, ModelView):
             `quantity` is mandatory information which needs to be returned, no
             matter what your logic for computing that is
 
-        :param product: ID of the product
         :return: A dictionary with `quantity` and `forecast_quantity`
         """
         context = {
@@ -92,13 +91,14 @@ class Product(ModelSQL, ModelView):
         }
         with Transaction().set_context(**context):
             return {
-                'quantity': self.get_quantity(
-                    [product], 'quantity')[product],
-                'forecast_quantity': self.get_quantity(
-                    [product], 'forecast_quantity')[product],
+                'quantity':
+                    self.get_quantity([self], 'quantity')[self.id],
+                'forecast_quantity':
+                    self.get_quantity([self], 'forecast_quantity')[self.id],
             }
 
-    def availability(self, uri):
+    @classmethod
+    def availability(cls, uri):
         """
         Returns the following information for a product:
 
@@ -115,21 +115,20 @@ class Product(ModelSQL, ModelView):
             stock with your vendor for back orders or send a message like
             `Only 5 pieces left`
 
-        :param product: ID of the product
+        :param uri: URI of the product for which the availability needs to
+                    be found
         :return: JSON object
         """
         allowed_categories = request.nereid_website.get_categories() + [None]
-        product_ids = self.search([
-            ('displayed_on_eshop', '=', True),
-            ('uri', '=', uri),
-            ('category', 'in', allowed_categories),
-            ]
-        )
-        if not product_ids:
+
+        try:
+            product, = cls.search([
+                ('displayed_on_eshop', '=', True),
+                ('uri', '=', uri),
+                ('category', 'in', allowed_categories),
+                ]
+            )
+        except ValueError:
             return abort(404)
 
-        # Location of stock for the website
-        location = request.nereid_website.stock_location.id
-        return jsonify(self.get_availability(product_ids[0]))
-
-Product()
+        return jsonify(product.get_availability())
