@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 #This file is part of Tryton and Nereid.  The COPYRIGHT file at the top level of
 #this repository contains the full copyright notices and license terms.
-import unittest2 as unittest
+import unittest
 
 from test_cart import TestCart
 from test_product import TestProduct
